<?php

namespace FM\Service;

use FM\Factory\DashboardFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\FuncionariosHierarqService;

class DashboardService
{

    protected $connection;
    protected $DashboardFactory;
    protected $FuncionariosHierarqService;
    protected $Environment;
    protected $newCode;

    public function __construct(EntityManager $entityManager, DashboardFactory $DashboardFactory, FuncionariosHierarqService $FuncionariosHierarqService, Environment $Environment, NewCode $newCode)
    {
        $this->connection                   = $entityManager->getConnection();
        $this->DashboardFactory             = $DashboardFactory;
        $this->FuncionariosHierarqService   = $FuncionariosHierarqService;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
    }

    public function getDadosLimpezaFinalizada()
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        return $this->DashboardFactory->getDadosLimpezaFinalizada($listaOperadores);
    }

    public function getDadosTarefaAtrasada()
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        return $this->DashboardFactory->getDadosTarefaAtrasada($listaOperadores);
    }

    public function getDadosTarefaConcluida()
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        return $this->DashboardFactory->getDadosTarefaConcluida($listaOperadores);
    }

    public function getDadosProgress()
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        $countTarefasConcluidas = $this->DashboardFactory->countTarefasConcluidas($listaOperadores);
        $countTarefas  = $this->DashboardFactory->countTarefas($listaOperadores);
        return [$countTarefasConcluidas, $countTarefas];
    }

    public function getDadosUltimasTarefas()
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        return $this->DashboardFactory->getDadosUltimasTarefas($listaOperadores);
    }

    public function getDadosResumoTarefas()
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        return $this->DashboardFactory->getDadosResumoTarefas($listaOperadores);
    }

    public function findLocal()
    {
        return $this->DashboardFactory->findLocal();
    }

    public function getDadosChartTarefas()
    {
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($this->Environment->getCdOperador());
        return $this->DashboardFactory->getDadosChartTarefas($listaOperadores);
    }

    public function getDadosChartNaoConformidades()
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        return $this->DashboardFactory->getDadosChartNaoConformidades($listaOperadores);
    }

    public function getDadosChartProgressoMensal()
    {
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($this->Environment->getCdOperador());
        return $this->DashboardFactory->countTarefasConcluidasMensal($listaOperadores);
    }
}
