/**
 * Nome: Heitor Medeiros
 * Data: 15/05/2023
 * Descrição: Tela responsável por criar Habilidade para os operadores. Posteriormente essas habilidaes serão
 * vinculadas às atividades. A Atividade só poderá ser realizada pelo operador que tiver a habilidade necessária.
 * Autor: Heitor Medeiros
 */
import { Utilities } from '@/util/Utilities';
import {
    DialogService, LoadingService, Tabs, TextInput,
} from '@zeedhi/common';
import { Http, IEventParam, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class Fm00000000HabilidadesController {
    // FUNÇÕES FRAME GRID HABILIDADES
    public addNewRow({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRHABILIDADE: newId, IDATIVO: 'S' }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`NMHABILIDADE_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    public salvarHabilidades({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            Http.post('/salvarHabilidades', {
                ROW: component.getEditedRows(),
            }).then(() => {
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                component.cancelEditedRows();
                LoadingService.hide();
            });
        }
    }

    public excluirHabilidades({ event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        const grid = Metadata.getInstance<TekGrid>('gridHabilidades');
        LoadingService.show();
        Http.post('/excluirHabilidades', {
            ROW: grid.selectedRows.map((element: any) => element.NRHABILIDADE),
        }).then(() => {
            grid.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            grid.selectedRows = [];
            LoadingService.hide();
            DialogService.hide();
        });
    }

    // FUNÇÕES TAB OPERADORES
    public addNewRowOperador({ component }: IEventParam<TekGrid>) {
        this.selectEditavel = true;
        const newId = 0;
        component.addNewRow({ NRRELACHABOPER: newId }, 'start');
    }

    public salvarOperadorHabilidades({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            const newRow = component.getEditedRows();
            Http.post('/relacionarOperadorHabilidade', {
                OPERADOR: newRow[0].CDOPERADOR,
                NRHABILIDADE: this.currentRow.NRHABILIDADE,
            }).then(() => {
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                this.selectEditavel = false;
                component.cancelEditedRows();
                LoadingService.hide();
            });
        }
    }

    public filtroOperadorHabilidade() {
        const gridOperador = Metadata.getInstance<TekGrid>('gridOperador');
        gridOperador.datasource.setFilter({
            HABILIDADE: this.currentRow.NRHABILIDADE,
        });
    }

    public excluirOperadorHabilidades({ event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        const grid = Metadata.getInstance<TekGrid>('gridOperador');
        LoadingService.show();
        Http.post('/excluirOperadorHabilidades', {
            ROW: grid.selectedRows.map((element: any) => element.NRRELACHABOPER),
        }).then(() => {
            grid.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            grid.selectedRows = [];
            LoadingService.hide();
            DialogService.hide();
        });
    }

    // FUNÇÕES TAB ATIVIDADES
    public addNewRowAtividade({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRRELACHABATIV: newId }, 'start');
    }

    public filtroAtividadeHabilidade() {
        const gridAtividade = Metadata.getInstance<TekGrid>('gridAtividades');
        gridAtividade.datasource.setFilter({
            HABILIDADE: this.currentRow.NRHABILIDADE,
        });
    }

    public salvarAtividadeHabilidades({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            const newRow = component.getEditedRows();
            Http.post('/relacionarAtividadeHabilidade', {
                NRTAREFA: newRow[0].NMTAREFA,
                NRHABILIDADE: this.currentRow.NRHABILIDADE,
            }).then(() => {
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                component.cancelEditedRows();
                LoadingService.hide();
            });
        }
    }

    public excluirAtividadeHabilidades({ event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        const grid = Metadata.getInstance<TekGrid>('gridAtividades');
        LoadingService.show();
        Http.post('/excluirAtividadeHabilidades', {
            ROW: grid.selectedRows.map((element: any) => element.NRRELACHABATIV),
        }).then(() => {
            grid.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            grid.selectedRows = [];
            LoadingService.hide();
            DialogService.hide();
        });
    }

    // CONTROLE DAS TELAS
    private clicks: any = 0;

    private currentRow: any;

    public visibilidadeFrameGridPincipal: Boolean = true;

    public visibilidadeFrameOperadoresAtividades: Boolean = false;

    public selectEditavel: Boolean = false;

    public tituloHeaderFrameOperadoresAtividades: String = '';

    public recarregarTabs() {
        const setOfTabs = Metadata.getInstance<Tabs>('tabsHabilidades');

        if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tabAtividades') {
            setTimeout(() => {
                this.filtroAtividadeHabilidade();
            }, 1500);
        } else if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tabOperadores') {
            setTimeout(() => {
                this.filtroOperadorHabilidade();
            }, 1500);
        }
    }

    public rowClick({ row }: any) {
        this.clicks += 1;
        setTimeout(() => {
            if (this.clicks === 1) {
                this.currentRow = row;
                this.visibilidadeFrameOperadoresAtividades = true;
                this.visibilidadeFrameGridPincipal = false;
                this.tituloHeaderFrameOperadoresAtividades = row.NMHABILIDADE;
                this.recarregarTabs();
            }
            this.clicks = 0;
        }, 1000);
    }

    public voltarGridHabilidades() {
        this.visibilidadeFrameOperadoresAtividades = false;
        this.visibilidadeFrameGridPincipal = true;
    }
}
