import {
    FileInput, Form, LoadingService, Modal, ModalService, Select, TextInput, Textarea,
} from '@zeedhi/common';
import {
    Http, IDictionary, Metadata,
} from '@zeedhi/core';
import {
    IterableCarousel, IterableComponentRender, TekGrid,
} from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';
import FileUpload from './FileUpload';

export class FmtOrdemServicoController {
    private visibilidade: boolean = true;

    private visibilidadeFrame: boolean = false;

    private modalCriarOrdemServico!: Modal;

    private carouselOrdemVisivel: boolean = false;

    private isMountedComentarios: boolean = false;

    private currentRow: any;

    public mostrarDetalhesOrdem({ row }: any) {
        const inpEquipamento = Metadata.getInstance<TextInput>('input-equipamento-tab-detalhes');
        const inpCategoria = Metadata.getInstance<TextInput>('input-categoria-tab-detalhes');
        const inpTipo = Metadata.getInstance<TextInput>('input-tipo-manut-tab-detalhes');
        const inpPrioridade = Metadata.getInstance<TextInput>('input-prioridade-tab-detalhes');
        const inpStatus = Metadata.getInstance<TextInput>('input-status-tab-detalhes');
        const inpData = Metadata.getInstance<TextInput>('input-data-tab-detalhes');
        const inpEscopo = Metadata.getInstance<TextInput>('input-escopo-tab-detalhes');
        const txtDescricao = Metadata.getInstance<Textarea>('textarea-descricao-tab-detalhes');

        inpEquipamento.value = row.NMEQUIPAMENTO;
        inpCategoria.value = row.NMCATEGORIAMANUT;
        inpTipo.value = row.NMTIPOMANUTENCAO;
        inpPrioridade.value = row.NMPRIORIDADEMANUT;
        inpStatus.value = row.NMIDSITUACAO;
        inpData.value = row.DTORDEMSERV;
        inpEscopo.value = row.NMESCOPOMANUT;
        txtDescricao.value = row.DSORDEMSERV;

        this.visibilidade = false;
        this.visibilidadeFrame = !this.visibilidade;
        this.currentRow = row;

        setTimeout(() => {
            if (this.isMountedComentarios) {
                const cardsComentarios = Metadata.getInstance<IterableComponentRender>('iterable-cards-comentarios');
                this.carregarComentariosOrdem({ component: cardsComentarios });
            }
        }, 300);
    }

    public carregarComentariosOrdem({ component }: any) {
        this.getComentariosOrdem(this.currentRow.NRORDEMSERV).then((response: any) => {
            component.datasource.data = [];
            if (response.data.data.length > 0) {
                response.data.data.forEach((dados: IDictionary) => {
                    component.datasource.data.push({
                        NRCOMENTORDEMSERV: dados.NRCOMENTORDEMSERV,
                        NMFUNCIONARIO: dados.NMFUNCIONARIO,
                        DTCOMENTARIO: dados.DTCOMENTARIO,
                        DSCOMENTARIO: dados.DSCOMENTARIO,
                    });
                });
            }
        });
    }

    public getComentariosOrdem(NRROW: any) {
        return Http.post('/getComentariosOrdem', {
            NRORDEMSERV: NRROW,
        });
    }

    public montaDadosComentarios() {
        const cardsComentarios = Metadata.getInstance<IterableComponentRender>('iterable-cards-comentarios');
        this.carregarComentariosOrdem({ component: cardsComentarios });
        this.isMountedComentarios = true;
    }

    public saveComentarioOrdem() {
        const form = Metadata.getInstance<Form>('form-comentario-tab');

        if (Utilities.validateForm(form)) {
            Http.post('/saveComentarioOrdem', {
                NRORDEMSERV: this.currentRow.NRORDEMSERV,
                DSCOMENTARIO: form.value.textareaComentarios,
            }).then(() => {
                Utilities.showSuccessMessage();
                Metadata.getInstance<Textarea>('textareaComentarios').value = null;
                const cardsComentarios = Metadata.getInstance<IterableComponentRender>('iterable-cards-comentarios');
                this.carregarComentariosOrdem({ component: cardsComentarios });
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            });
        }
    }

    public voltarTelaOrdemServico() {
        this.visibilidade = true;
        this.visibilidadeFrame = !this.visibilidade;
    }

    public async criarOrdemServico() {
        this.modalCriarOrdemServico?.destroy();
        this.modalCriarOrdemServico = await ModalService.createFromJsonPost('', { jsonPath: 'fmt_ordem_servico/modal/fmt_criar_ordem_servico' });
        this.modalCriarOrdemServico.show();
    }

    public fecharModalCriarOrdem() {
        this.modalCriarOrdemServico?.destroy();
        this.modalCriarOrdemServico.hide();
    }

    public mudaVisibilidadeTecnicos({ component }: any) {
        const selectTecnicos = Metadata.getInstance<Select>('selectTecnicos');
        selectTecnicos.isVisible = component.value === 'N';
    }

    public async saveOrdemServico() {
        const form = Metadata.getInstance<Form>('form-criar-ordem-servico');
        const imgs = await this.updloadPictures('file-input-fotos-ordem');
        const grid = Metadata.getInstance<TekGrid>('grid-ordem-servico');

        if (Utilities.validateForm(form)) {
            LoadingService.show();
            Http.post('/saveOrdemServico', {
                NMORDEMOSERV: form.value.inputNome,
                IDSITUACAO: form.value.selectStatus,
                NREQUIPAMENTO: form.value.selectEquipamento,
                NRCATEGORIAMANUT: form.value.selectCategoria,
                NRTIPOMANUTENCAO: form.value.selectTipoManutencao,
                NRPRIORIDADEMANUT: form.value.selectPrioridade,
                DTORDEMSERV: form.value.dateData,
                IDFUNCEXTERNO: form.value.selectProfiExterno,
                TECNICOS: form.value.selectTecnicos,
                DSORDEMSERV: form.value.textDescricao,
                NRESCOPOS: form.value.selectEscopo,
                FOTOS: imgs,
                DSCOMENTARIO: form.value.textComentario,
                SOLICITACAO: false,
            }).then(() => {
                grid.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                this.fecharModalCriarOrdem();
                LoadingService.hide();
            });
        }
    }

    public mostrarFotoOrdem({ component }: any) {
        const carouselFotos = Metadata.getInstance<IterableCarousel>('carouselFotosOrdem');
        carouselFotos.iterable.datasource.updateData([]);
        const imagens: any = [];
        component.value.forEach((element: any, pos: any) => {
            imagens.push({
                id: `numero_${pos}`,
                src: window.URL.createObjectURL(element),
            });
        });
        carouselFotos.iterable.datasource.updateData(imagens);
        this.carouselOrdemVisivel = component.value.length > 0;
    }

    public async updloadPictures(name: string) {
        const fileInputObject = Metadata.getInstance<FileInput>(name).value;
        let arquivos: any = [];
        if (fileInputObject && fileInputObject.length > 0) {
            const types: IDictionary = ['image/jpg', 'image/jpeg', 'image/png'];
            const buildFilesPromises = fileInputObject.map((file: any) => {
                if (types.includes(file.type)) {
                    return FileUpload.buildFileSendObject(file);
                }
                return [];
            });
            await Promise.all(buildFilesPromises).then((files: any) => {
                arquivos = files;
            });
          }
        return arquivos;
    }
}
